package videoautomat;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import log.LogInputStream;
import sale.Action;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.Gate;
import sale.GateChangeTransition;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.UIGate;
import sale.stdforms.MsgForm;
import videoautomat.gui.LogFileFormSheet;
/**
 * This class implements a <code>SaleProcess</code> for the administrative work.
 *  
 */
public class SaleProcessAdmin extends SaleProcess {
	/*
	 * the initial Gate, where the global logfile is shown.
	 */
	private UIGate uig_logfile = new UIGate(null, null);

	/**
	 * Constructs a new SaleProcessAdmin.
	 *  
	 */
	public SaleProcessAdmin() {
		super("SaleProcessAdmin");
	}
	/**
	 * Implementation of the inherited abstract method. At this <code>Gate</code> the user will see the content of
	 * the global logfile.
	 * 
	 * @return the <code>Gate</code> this process will first switch to.
	 * 
	 * @see sale.SaleProcess#getInitialGate()
	 */
	protected Gate getInitialGate() {
		try {
			FileInputStream fis = new FileInputStream(VideoShop.FILENAME);
			LogInputStream lis = new LogInputStream(fis, new LogEntryFilterImpl());

			LogFileFormSheet lfsf = new LogFileFormSheet(lis);
			lfsf.addContentCreator(new FormSheetContentCreator() {
				public void createFormSheetContent(FormSheet fs) {
					fs.getButton(LogFileFormSheet.FB_CLOSE).setAction(new Action() {
						public void doAction(SaleProcess p, SalesPoint sp) {
							uig_logfile.setNextTransition(
								GateChangeTransition.CHANGE_TO_STOP_GATE);
						}
					});
				}
			});
			uig_logfile.setFormSheet(lfsf);
		} catch (FileNotFoundException e) {
			getStopGate();
			e.printStackTrace();
		} catch (IOException e) {
			MsgForm msf = new MsgForm("Empty-LogFile!",
				"The log file was found empty.\n"
				+ "Nothing to administrate here.");
			msf.getButton(MsgForm.BTNID_OK).setAction(new Action() 
			{
				public void doAction(SaleProcess p, SalesPoint sp) 
				{
					uig_logfile.setNextTransition(GateChangeTransition.CHANGE_TO_QUIT_GATE);
				}});
			uig_logfile.setFormSheet(msf);
		}
		return uig_logfile;
	}
}
