package videoautomat;

import java.util.Iterator;

import sale.*;
import users.User;
import videoautomat.gui.HandBack;
import data.IntegerValue;
import data.NotEnoughMoneyException;
import data.NumberValue;
import data.QuoteValue;
import data.StockItem;
import data.StoringStock;
import data.Value;
import data.ooimpl.MoneyBagImpl;
import data.ooimpl.StoringStockImpl;
import data.stdforms.TwoTableFormSheet;
import data.stdforms.twotableformsheet.SSSSStrategy;

/**
 * This class implements a <code>SaleProcess</code> to hand back the videos.
 *  
 */
public class SaleProcessHandBack extends SaleProcess {

	/*
	 * The initial Gate of this process at which the user can choose from his/her rented videos.
	 */
	private UIGate uig_video = new UIGate(null, null);

	/*
	 * The Gate where the user will get informed about his/her change money.
	 */
	private UIGate uig_change = new UIGate(null, null);

	/*
	 * A Stock to store temporary videos.
	 */
	private StoringStock ss_temp =
		new StoringStockImpl("temp", VideoShop.getVideoCatalog());

	/*
	 * A MoneyBag to store temporary the change money
	 */
	private MoneyBagImpl mb_temp = new MoneyBagImpl("temp", VideoShop.getCurrency());

	/*
	 * A NumberValue representing the change money
	 */
	private NumberValue nv_sum = new IntegerValue(0);

	/**
	 * Constructs a new <code>SaleProcessHandBack</code>
	 *  
	 */
	public SaleProcessHandBack() {
		super("SaleProcessGiveback");
	}

	/**
	 * Implementation of the inherited abstract method.
	 * 
	 * @return the <code>Gate</code> where the user will see his/her current rented videos.
	 * 
	 * @see sale.SaleProcess#getInitialGate()
	 */
	protected Gate getInitialGate() {
		TwoTableFormSheet ttfs =
			HandBack.getRentedVideosFormSheet(
				((AutomatUser) ((SalesPoint) this.getContext()).getUser()).getVideoStock(),
				ss_temp,
				getBasket(),
				uig_video,
				null,
				null,
				new TEDVideoCassette(),
				new TEDVideoCassette(),
				new SSSSStrategy());

		ttfs.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				fs.getButton(HandBack.FB_GIVEBACK).setAction(new Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						uig_video.setNextTransition(getCalculateSumTransition());
					}
				});
				fs.getButton(HandBack.FB_CANCEL).setAction(new Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						uig_video.setNextTransition(
							new GateChangeTransition(getRollbackGate()));
					}
				});
			}
		});

		uig_video.setFormSheet(ttfs);
		return uig_video;
	}

	/**
	 * @return the <code>Gate</code> where the user will see his/her change money.
	 *  
	 */
	private Gate getChangeGate() {
		String s = VideoShop.getCurrency().toString(nv_sum);
		FormSheet fs = HandBack.getChangeFormSheet(mb_temp, uig_change, getBasket(), s);

		fs.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				fs.getButton(FormSheet.BTNID_OK).setAction(new Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						uig_change.setNextTransition(
							GateChangeTransition.CHANGE_TO_COMMIT_GATE);
					}
				});
			}
		});

		uig_change.setFormSheet(fs);
		return uig_change;
	}

	/**
	 * @return a <code>Transition</code> that calculates the money the user get back, temporary re-adds the handed
	 *              back videos to the {@link VideoShop}s stock and change to the
	 *              {@link SaleProcessHandBack#getChangeGate()}. If there is not enough change money in the shop, a
	 *              {@link DisplayMoneyStockError}opens and it returns the sale.SaleProcess#getRollbackGate()
	 *  
	 */
	private Transition getCalculateSumTransition() {
		return new Transition() {
			public Gate perform(SaleProcess p, User u) {
				if (ss_temp.size(getBasket()) == 0)
					return getInitialGate();

				Iterator it = ss_temp.iterator(getBasket(), false);
				while (it.hasNext()) {
					StockItem ssi = (StockItem) it.next();
					Value v = null;
					v = ((QuoteValue) ssi.getAssociatedItem(null).getValue()).getOffer();
					nv_sum =
						(NumberValue) nv_sum.add(
							v.subtract(((VideoCassette) ssi).getCost()));

					VideoShop.getVideoStock().add(ssi.getName(), 1, getBasket());
				}
				try{
					VideoShop.getMoneyBag().transferMoney(mb_temp, getBasket(), nv_sum);
				}catch(NotEnoughMoneyException e){
					DisplayMoneyStockError dmse = new DisplayMoneyStockError();
					return getRollbackGate();
				}
				
				return getChangeGate();
			}
		};
	}
}
