package videoautomat;
import sale.*;
import users.ActionCapability;
import users.User;
import users.UserManager;
import users.stdforms.LogOnForm;
import videoautomat.gui.LogOn;
import data.ooimpl.DataBasketImpl;

/**
 * This class implements a <code>SaleProcess</code> used to log on and choose which activity should follow.
 *  
 */
public class SaleProcessLogOn extends SaleProcess {
	
	/*
	 * The initial Gate of this process, at which the user selects it`s ID and type in the password
	 */
	private UIGate uig_log_on = new UIGate(null, null);

	/*
	 * This Gate is set if the user selected no name or typed in the wrong password
	 */
	private UIGate uig_fault = new UIGate(null, null);

	/*
	 * At this Gate the user selects his/her further activities e.g. renting/handing back a video or administrative
	 * stuff
	 */
	private UIGate uig_main = new UIGate(null, null);
	
	/*
	 * A LogOnForm used for the uig_log_on
	 */
	private LogOnForm lof_initial = null;
	/*
	 * Refers to the current user
	 */
	private User u_current = null;
	/**
	 * Constructs a new <code>SaleProcessLogOn</code>
	 *  
	 */
	public SaleProcessLogOn() {
		super("SaleProcessLogOn");
	}

	/**
	 * Implementation of the inherited abstract method.
	 * 
	 * @return a <code>Gate</code> where the user selects it`s user-name and is asked for his/her password.
	 * 
	 * @see sale.SaleProcess#getInitialGate()
	 */
	protected Gate getInitialGate() {
		lof_initial = LogOn.getLogOnForm(true, UserManager.getGlobalUM(), null, null);
		lof_initial.addContentCreator(new FormSheetContentCreator() {
			protected void createFormSheetContent(FormSheet sheet) {
				sheet.getButton(FormSheet.BTNID_OK).setAction(new sale.Action() {
					public void doAction(SaleProcess process, SalesPoint point) {
						uig_log_on.setNextTransition(logOn());
					}
				});
				sheet.getButton(FormSheet.BTNID_CANCEL).setAction(new sale.Action() {
					public void doAction(SaleProcess process, SalesPoint point) {
						uig_log_on.setNextTransition(
							GateChangeTransition.CHANGE_TO_STOP_GATE);
					}
				});
			}
		});
		uig_log_on.setFormSheet(lof_initial);
		return uig_log_on;
	}
	/**
	 * @return a <code>Gate</code> that shows an error-message.
	 */
	private Gate getFaultGate() {
		FormSheet fs_fault = LogOn.getFaultFormSheet();

		fs_fault.addContentCreator(new FormSheetContentCreator() {
			protected void createFormSheetContent(FormSheet sheet) {
				sheet.getButton(FormSheet.BTNID_OK).setAction(new sale.Action() {
					public void doAction(SaleProcess process, SalesPoint point) {
						uig_fault.setNextTransition(
							new GateChangeTransition(getInitialGate()));
					}
				});
			}
		});

		uig_fault.setFormSheet(fs_fault);
		return uig_fault;
	}
	/**
	 * @return a <code>Gate</code> where the user can select the next activity, like renting a video.
	 */
	private Gate getMainGate() {
		FormSheet fs_main =
			LogOn.getMainFormSheet(
				VideoShop.getVideoStock(),
				uig_main,
				false,
				new TEDVideoStock());
		fs_main.addContentCreator(new FormSheetContentCreator() {
			protected void createFormSheetContent(FormSheet fs) {
				fs.getButton(LogOn.FB_RENT).setAction(new sale.Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						sp.runProcess(new SaleProcessRent(), new DataBasketImpl());
					}
				});
				fs.getButton(LogOn.FB_HANDBACK).setAction(new sale.Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						sp.runProcess(new SaleProcessHandBack(), new DataBasketImpl());
					}
				});
				fs.getButton(LogOn.FB_ADMIN).setAction(
					(ActionCapability) u_current.getCapability(
						AutomatUser.CAPABILITY_ADMIN));
				fs.getButton(LogOn.FB_LOGOUT).setAction(new sale.Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						sp.detachUser();
						uig_main.setNextTransition(
							GateChangeTransition.CHANGE_TO_STOP_GATE);
					}
				});
			}
		});
		uig_main.setFormSheet(fs_main);
		return uig_main;
	}
	/**
	 * @return a <code>Transition</code> that proves the selected name and password and with success leads to the
	 *              {@link SaleProcessLogOn#getMainGate()}, otherwise it returns the
	 *              {@link SaleProcessLogOn#getFaultGate()}.
	 */
	private Transition logOn() {
		return new Transition() {
			public Gate perform(SaleProcess process, User user) {
				lof_initial.ok();
				u_current = lof_initial.getResult();
				if (u_current != null) {
					((SalesPoint) process.getContext()).attach(u_current);					
					return getMainGate();
				}
				return getFaultGate();
			}
		};
	}
}
