package videoautomat;
import java.util.Iterator;

import sale.*;
import sale.FormSheet.FormButton;
import sale.stdforms.FormSheetStrategy;
import users.User;
import videoautomat.gui.Rent;
import data.*;
import data.events.StockChangeAdapter;
import data.events.StockChangeEvent;
import data.events.VetoException;
import data.ooimpl.CountingStockItemDBEntry;
import data.ooimpl.MoneyBagImpl;
import data.stdforms.TwoTableFormSheet;
import data.stdforms.twotableformsheet.CCSStrategy;
import data.stdforms.twotableformsheet.CSDBStrategy;
import data.swing.DefaultStoringStockDBETableEntryDescriptor;
/**
 * This class implements a <code>SaleProcess</code> used to rent videos.
 *  
 */
public class SaleProcessRent extends SaleProcess {
	/*
	 * Key of a Databasket-subbasket which contains the temporary removed videos of the VideoShops stock
	 */
	private static final String SUB_SHOP_VIDEO = "videos_cs";
	/*
	 * Key of Databasket-subbasket which contains the temporary added videos of the users-stock
	 */
	private static final String SUB_USER_VIDEO = "video_ss";
	/*
	 * Key of Databasket-subbasket which contains the temporary added money of the user
	 */
	private static final String SUB_TMP_MONEY = "money_temp";
	/*
	 * Key of Databasket-subbasket which contains the temporary removed money of the VideoShops MoneyBag
	 */
	private static final String SUB_SHOP_MONEY = "money_shop";
	/*
	 * The inital Gate where the offer of this automat is presented
	 */
	private UIGate uig_offer = new UIGate(null, null);

	/*
	 * The Gate where the user has to pay for the selected videos
	 */
	private UIGate uig_pay = new UIGate(null, null);

	/*
	 * The Gate which shows what the user has rented and how much change he/she gets
	 */
	private UIGate uig_confirm = new UIGate(null, null);
	/*
	 * A MoneyBag used to temporary hold the money in it, therefor not the whole money of the VideoShop is visible
	 */
	private MoneyBagImpl mb_temp =
		new MoneyBagImpl("mb_user", VideoShop.getCurrency());

	/*
	 * A NumberValue used to store the sum the user has to pay
	 */
	private NumberValue nv_sum = null;
	/**
	 * Constructs a new SaleProcessRent
	 *  
	 */
	public SaleProcessRent() {
		super("SaleProcessRent");
	}
	/**
	 * Implementation of the inherited abstract method.
	 * 
	 * @return a <code>Gate</code> where the user makes a selection
	 * 
	 * @see sale.SaleProcess#getInitialGate()
	 */
	protected Gate getInitialGate() {
		getBasket().setCurrentSubBasket(SUB_SHOP_VIDEO);
		CSDBStrategy csdbs = new CSDBStrategy();
		csdbs.setErrorHandler(FormSheetStrategy.MSG_POPUP_ERROR_HANDLER);
		TwoTableFormSheet ttfs_rent =
			Rent.getRentFormSheet(
				VideoShop.getVideoStock(),
				getBasket(),
				uig_offer,
				null,
				null,
				false,
				new TEDVideoStock(),
				null,
				csdbs);

		ttfs_rent.addContentCreator(new FormSheetContentCreator() {
			protected void createFormSheetContent(FormSheet fs) {
				fs.getButton(Rent.FB_RENT).setAction(new sale.Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						uig_offer.setNextTransition(getSumUpTransition());
					}
				});
				fs.getButton(Rent.FB_CANCEL).setAction(new sale.Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						uig_offer.setNextTransition(
							GateChangeTransition.CHANGE_TO_ROLLBACK_GATE);
					}
				});

			}
		});
		return uig_offer;
	}
	/**
	 * @return a <code>Gate</code> where the money gets inserted
	 */
	private Gate getPayGate() {
		String value = VideoShop.getCurrency().toString(nv_sum);
		CCSStrategy ccss = new CCSStrategy();
		ccss.setErrorHandler(FormSheetStrategy.MSG_POPUP_ERROR_HANDLER);
		TwoTableFormSheet ttfs_pay =
			Rent.getPayFormSheet(
				VideoShop.getCurrency(),
				mb_temp,
				getBasket(),
				uig_pay,
				new ComparatorCurrency(),
				new ComparatorCurrency(),
				false,
				null,
				null,
				ccss,
				value);
		ttfs_pay.addContentCreator(new FormSheetContentCreator() {
			protected void createFormSheetContent(final FormSheet fs) {
				fs.getButton(Rent.FB_PAY).setAction(new sale.Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						uig_pay.setNextTransition(getPayConfirmTransition());
					}
				});
				fs.getButton(Rent.FB_PAY).setEnabled(false);
				setButtonStockListener(fs.getButton(Rent.FB_PAY));
				fs.getButton(Rent.FB_CANCEL).setAction(new sale.Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						uig_pay.setNextTransition(getPayRollbackTransition());
					}
				});
			}
		});
		return uig_pay;
	}
	/**
	 * @return a <code>Gate</code> where the selected videos and the change money is shown
	 */
	private Gate getConfirmGate() {
		FormSheet fs =
			Rent.getConfirmFormSheet(
				getBasket(),
				DataBasketConditionImpl.allStockItemsWithDest(
					((AutomatUser) ((SalesPoint) getContext()).getUser()).getVideoStock()),
				new DefaultStoringStockDBETableEntryDescriptor(),
				mb_temp,
				null);
		fs.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				fs.getButton(FormSheet.BTNID_OK).setAction(new Action() {
					public void doAction(SaleProcess p, SalesPoint sp) {
						uig_confirm.setNextTransition(
							GateChangeTransition.CHANGE_TO_COMMIT_GATE);
					}
				});
			}
		});

		uig_confirm.setFormSheet(fs);
		return uig_confirm;
	}
	/**
	 * @return a <code>Transition</code> that sums up the prices of the selected videos and leads to the
	 *              {@link SaleProcessRent#getPayGate()}, if no video is selected it leads to the
	 *              {@link SaleProcessRent#getInitialGate()}
	 */
	private Transition getSumUpTransition() {
		return new Transition() {
			public Gate perform(SaleProcess p, User u) {
				nv_sum = (NumberValue) getBasket().sumSubBasket(SUB_SHOP_VIDEO,
						null, new BasketEntryValue() {
							public Value getEntryValue(DataBasketEntry dbe) {
								try {
									CatalogItem ci = VideoShop
											.getVideoCatalog().get(
													dbe.getSecondaryKey(),
													null, false);
									int count = ((Integer) dbe.getValue())
											.intValue();
									return ((QuoteValue) ci.getValue())
											.getOffer().multiply(count);
								} catch (VetoException e) {
									e.printStackTrace();
									getBasket().rollback();
								}
								return null;
							}
						}, new IntegerValue(0));
				if (nv_sum.isAddZero())
					return getInitialGate();
				getBasket().setCurrentSubBasket(SUB_TMP_MONEY);
				return getPayGate();
			}
		};
	}
	/**
	 * @return a <code>Transition</code> that rollback the money-transactions and leads back to the
	 *              {@link SaleProcessRent#getInitialGate()}
	 */
	private Transition getPayRollbackTransition() {
		return new Transition() {
			public Gate perform(SaleProcess p, User u) {
				getBasket().rollbackSubBasket(SUB_TMP_MONEY);
				return getInitialGate();
			}
		};
	}
	/**
	 * @return a <code>Transition</code> that temporary adds the selected videos to the
	 *              {@link AutomatUser#getVideoStock()}, the inserted money to the {@link VideoShop#getVideoStock()}and from
	 *              there transacts the change money
	 */
	private Transition getPayConfirmTransition() {
		return new Transition() {
			public Gate perform(SaleProcess p, User u) {
				/*
				 * first add new rent-cassettes to the user`s stock
				 */
				getBasket().setCurrentSubBasket(SUB_USER_VIDEO);
				StoringStock ss_user =
					((AutomatUser) ((SalesPoint) p.getContext()).getUser())
						.getVideoStock();
				Iterator i =
					getBasket().subBasketIterator(
						SUB_SHOP_VIDEO,
						DataBasketConditionImpl.ALL_ENTRIES);
				while (i.hasNext()) {
					VideoCassette vc =
						new VideoCassette(
							((CountingStockItemDBEntry) i.next())
								.getSecondaryKey());
					ss_user.add(vc, getBasket());
				}
				/*
				 * calculate what is in the temporar moneybag
				 */
				getBasket().setCurrentSubBasket(SUB_TMP_MONEY);
				NumberValue nv =
					(NumberValue)
						(
							(NumberValue) mb_temp.sumStock(
								getBasket(),
								new CatalogItemValue(),
								new IntegerValue(0))).subtract(
						nv_sum);
				/*
				 * this prevents an exception that is caused by removing all items from the table
				 * while it is shown
				 */
				try {
					p.getContext().setFormSheet(p, null);
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}
				/*
				 * put the content of the temporar moneybag to the shop`s one and get the change
				 */
				VideoShop.getMoneyBag().addStock(mb_temp, getBasket(), true);
				getBasket().setCurrentSubBasket(SUB_SHOP_MONEY);
				try{	
					VideoShop.getMoneyBag().transferMoney(mb_temp, getBasket(), nv);					
				}catch(NotEnoughMoneyException e){
					getBasket().rollbackSubBasket(SUB_USER_VIDEO);
					getBasket().rollbackSubBasket(SUB_TMP_MONEY);
					getBasket().rollbackSubBasket(SUB_SHOP_MONEY);
					DisplayMoneyStockError dmse = new DisplayMoneyStockError();
					getBasket().setCurrentSubBasket(SUB_TMP_MONEY);
					return getPayGate();
				}
				return getConfirmGate();
			}
		};
	}
	/**
	 * Adds an implementation of the <code>StockChangeListener</code> to the given button, so that this button gets
	 * enabled if there is enough money in the temporar-moneybag and otherwise it gets disabled
	 * 
	 * @param fb
	 *                  the <code>FormButton</code> the listener should en/disable
	 */
	private void setButtonStockListener(final FormButton fb) {
		StockChangeAdapter sca = new StockChangeAdapter() {
			public void addedStockItems(StockChangeEvent e) {
				if (mb_temp
					.sumStock(getBasket(), new CatalogItemValue(), new IntegerValue(0))
					.compareTo(nv_sum)
					>= 0)
					fb.setEnabled(true);
			}
			public void removedStockItems(StockChangeEvent e) {
				if (mb_temp
					.sumStock(getBasket(), new CatalogItemValue(), new IntegerValue(0))
					.compareTo(nv_sum)
					< 0)
					fb.setEnabled(false);
			}
		};
		mb_temp.addStockChangeListener(sca);
	}
}
