package videoautomat.gui;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.UIGate;
import sale.stdforms.MsgForm;
import util.swing.TableEntryDescriptor;
import data.CountingStock;
import data.stdforms.SingleTableFormSheet;

/**
 * This class implements graphical-user-interface elements for the whole application, especially
 * for the <code>VideoAutomat</code>
 */
public class Global {
	/**
	 * The Caption of the <code>VideoAutomat</code>
	 */
	public static final String CAPTION_AUTOMAT =
		"****** VIDEOAUTOMAT *** 24 H ******";
	/**
	 * Label for the admin- <code>ActionCapability</code>, if it`s not granted.
	 */
	public static final String MSG_ACCESS = "Acces denied!!!";
	/**
	 * Label for the menusheet that contains self-defined items
	 */
	public static final String MS_NEW = "Videoautomat";

	/**
	 * Label for the button to start the automat
	 */
	public static final String MSI_AUTOMAT = "Start automat";
	/**
	 * Label for the time setter button
	 */
	public static final String MSI_DAY = "+ 1 Day";
	/**
	 * An ID to identify the <code>FormButton</code> log on
	 */
	public static int FB_LOGON = 1;
	/**
	 * @return the {@link videoautomat.VideoAutomat#getDefaultFormSheet()}which displays the
	 *              video-offer and the possibility to start a {@link videoautomat.SaleProcessLogOn}
	 * @param cs
	 *                  the <Stock>containing the offer
	 * @param uigGate
	 *                  the <code>UIGate</code> this <code>FormSheet</code> should be diplayed at
	 * @param show_zeros
	 *                  if false, lines containing a '0' in the "Count" field will be hidden.
	 * @param ted
	 *                  the <code>TableEntryDescriptor</code> used to display the <Stock>
	 *  
	 */

	public static FormSheet getStartFormSheet(
		CountingStock cs,
		UIGate uigGate,
		boolean show_zeros,
		TableEntryDescriptor ted) {
		SingleTableFormSheet stfs_log =
			SingleTableFormSheet.create(
				"Please log on!",
				cs,
				uigGate,
				show_zeros,
				ted);
		stfs_log.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				fs.removeAllButtons();
				fs.addButton("Login", FB_LOGON, null);
			}
		});
		return stfs_log;
	}

	/**
	 * @return a <code>FormSheet</code> that informs the user that there isn`t the correct
	 *              amount of money left, to hand back the change.
	 */
	public static FormSheet getNoChangeFormSheet() {
		return new MsgForm(
			"No change!",
			"There is not enough change in here.\n"
				+ "Please insert the correct amount of money\n"
				+ "or contact the hotline.");
	}
}
