package videoautomat.gui;

import java.util.Comparator;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.UIGate;
import util.swing.TableEntryDescriptor;
import data.DataBasket;
import data.StoringStock;
import data.ooimpl.MoneyBagImpl;
import data.stdforms.SingleTableFormSheet;
import data.stdforms.TwoTableFormSheet;
import data.stdforms.twotableformsheet.SSSSStrategy;

/**
 * This class implements graphical-user-interface elements for the SaleProcessHandBack
 */
public class HandBack {

	/**
	 * An ID to identify the <code>FormButton</code> give back
	 */
	public static final int FB_GIVEBACK = 1;

	/**
	 * An ID to identify the <code>FormButton</code> cancel
	 */
	public static final int FB_CANCEL = 2;

	/**
	 * @return a <code>TwoTableFormSheet</code> used to display the already rented videos.
	 * @param ss_source
	 *                  the source <code>Stock</code> containing the videos of the user
	 * @param ss_dest
	 *                  the destination <code>Stock</code>
	 * @param db
	 *                  the <code>DataBasket</code> used for the transactions
	 * @param uig
	 *                  the <code>UIGate</code> this <code>FormSheet</code> is displayed
	 * @param cmp_source
	 *                  the <code>Comparator</code> for the source <code>Stock</code>
	 * @param cmp_dest
	 *                  the <code>Comparator</code> for the destination <code>Stock</code>
	 * @param ted_source
	 *                  the <code>TableEntryDescriptor</code> for the source <code>Stock</code>
	 * @param ted_dest
	 *                  the <code>TableEntryDescriptor</code> for the destination <code>Stock</code>
	 * @param sssss
	 *                  the <code>SSSSStrategy</code> to be used
	 *  
	 */
	public static TwoTableFormSheet getRentedVideosFormSheet(
		StoringStock ss_source,
		StoringStock ss_dest,
		DataBasket db,
		UIGate uig,
		Comparator cmp_source,
		Comparator cmp_dest,
		TableEntryDescriptor ted_source,
		TableEntryDescriptor ted_dest,
		SSSSStrategy sssss) {

		TwoTableFormSheet ttfs =
			TwoTableFormSheet.create(
				"Give back a video",
				ss_source,
				ss_dest,
				db,
				uig,
				cmp_source,
				cmp_dest,
				ted_source,
				ted_dest,
				sssss);

		ttfs.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				fs.removeAllButtons();
				fs.addButton("Give back", FB_GIVEBACK, null);
				fs.addButton("Cancel", FB_CANCEL, null);
			}
		});
		return ttfs;
	}

	/**
	 * @return a <code>FormSheet</code> used to display change the user gets back.
	 * @param mb_money
	 *                  the <code>MoneyBag</code> containing the change that has to be displayed
	 * @param uig
	 *                  the <code>UIGate</code> this <code>FormSheet</code> is displayed
	 * @param db
	 *                  the <code>DataBasket</code> used for the transactions
	 * @param value
	 *                  the <code>String</code> that will be displayed as the value that the user gets back
	 *  
	 */
	public static FormSheet getChangeFormSheet(
		MoneyBagImpl mb_money,
		UIGate uig,
		DataBasket db,
		final String value) {
		SingleTableFormSheet stfs =
			SingleTableFormSheet.create("Here is your change!", mb_money, uig, db);
		stfs.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				JComponent jc = new JPanel();
				jc.setLayout(new BoxLayout(jc, BoxLayout.Y_AXIS));
				jc.add(Box.createVerticalStrut(10));
				jc.add(new JLabel("You get back: " + value));
				jc.add(Box.createVerticalStrut(10));
				jc.add(fs.getComponent());
				fs.setComponent(jc);
				fs.removeButton(FormSheet.BTNID_CANCEL);
			}
		});
		return stfs;
	}
}
