package videoautomat.gui;
import java.util.Comparator;

import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.UIGate;
import sale.stdforms.MsgForm;
import users.UserManager;
import users.stdforms.LogOnForm;
import users.swing.UserFilter;
import util.swing.TableEntryDescriptor;
import data.CountingStock;
import data.stdforms.SingleTableFormSheet;

/**
 * This class implements graphical-user-interface elements for the SaleProcessLogOn
 */
public class LogOn {
	/**
	 * An ID to identify the <code>FormButton</code> rent
	 */
	public static int FB_RENT = 1;

	/**
	 * An ID to identify the <code>FormButton</code> hand back
	 */
	public static int FB_HANDBACK = 2;

	/**
	 * An ID to identify the <code>FormButton</code> admin
	 */
	public static int FB_ADMIN = 3;

	/**
	 * An ID to identify the <code>FormButton</code> logout
	 */
	public static int FB_LOGOUT = 4;
	/**
	 * @return a <code>LogOnForm</code>
	 * @param ask_password
	 *                  if false no password is needed and selection of the user name is sufficient.
	 * @param um
	 *                  the UserManager that manages the users to select from.
	 * @param cmp
	 *                  a comparator that defines the order in which the user names appear. If null, users will be ordered by
	 *                  their names.
	 * @param uf
	 *                  a filter that allows only a subset of the users to be selected from. If null, no filtering will
	 *                  occur.
	 *  
	 */
	public static LogOnForm getLogOnForm(
		boolean ask_password,
		UserManager um,
		Comparator cmp,
		UserFilter uf) {
		LogOnForm lof =
			new LogOnForm(
				"Are you a registered user?",
				"Select your user name!",
				"Enter your passphrase!",
				ask_password,
				um,
				cmp,
				uf);
		return lof;
	}

	/**
	 * @return a <code>FormSheet<code> informing the user, that logging on failed.
	 */
	public static FormSheet getFaultFormSheet() {
		return new MsgForm(
			"Log on failed!",
			"You didnt choose a user name or the passphrase didn`t match!");
	}

	/**
	 * @return a <code>FormSheet<code> where the video-offer is displayed and possibilities exist to start further activities.
	 * @param cs the <code>Stock</code> that contains the video-offer
	 * @param uigGate the <code>UIGate</code> this <code>FormSheet</code> is displayed
	 * @param show_zeros if false, lines containing a '0' in the "Count" field will be hidden.
	 * @param ted the <code>TableEntryDescriptor</code> used to display the <Stock>
	 *  
	 */
	public static FormSheet getMainFormSheet(
		CountingStock cs,
		UIGate uigGate,
		boolean show_zeros,
		TableEntryDescriptor ted) {
		SingleTableFormSheet stfs_main =
			SingleTableFormSheet.create(
				"Select an action!",
				cs,
				uigGate,
				show_zeros,
				ted);
		stfs_main.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				fs.removeAllButtons();
				fs.addButton("Rent", FB_RENT, null);
				fs.addButton("Hand back", FB_HANDBACK, null);
				fs.addButton("Administrate", FB_ADMIN, null);
				fs.addButton("Logout", FB_LOGOUT, null);
			}
		});

		return stfs_main;
	}
}
