package videoautomat.gui;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.UIGate;
import util.swing.TableEntryDescriptor;
import data.Catalog;
import data.CountingStock;
import data.DataBasket;
import data.DataBasketCondition;
import data.stdforms.SingleTableFormSheet;
import data.stdforms.TwoTableFormSheet;
import data.stdforms.twotableformsheet.CCSStrategy;
import data.stdforms.twotableformsheet.CSDBStrategy;

/**
 * This class implements graphical-user-interface elements for the SaleProcessRent
 */
public class Rent {
	
	/**
	 * An ID to identify the <code>FormButton</code> rent
	 */
	public static final int FB_RENT = 1;
	/**
	 * An ID to identify the <code>FormButton</code> pay
	 */
	public static final int FB_PAY = 2;
	/**
	 * An ID to identify the <code>FormButton</code> cancel
	 */
	public static final int FB_CANCEL = 3;
	/**
	 * @return a <code>TwoTableFormSheet</code> used to select the videos of the offer.
	 * @param cs_source
	 *                  the source <code>Stock</code> containing the avaible videos
	 * @param db
	 *                  the destination <code>DataBasket</code>
	 * @param uigGate
	 *                  the <code>UIGate</code> this <code>FormSheet</code> is displayed
	 * @param cmp_source
	 *                  the <code>Comparator</code> for the <code>Stock</code>
	 * @param cmp_dest
	 *                  the <code>Comparator</code> for the <code>DataBasket</code>
	 * @param show_zeros
	 *                  if false, lines containing a '0' in the "Count" field of the <code>Stock</code> will be hidden.
	 * @param ted_source
	 *                  the <code>TableEntryDescriptor</code> for the <code>Stock</code>
	 * @param ted_dest
	 *                  the <code>TableEntryDescriptor</code> for the <code>DataBasket</code>
	 * @param csdbs
	 *                  the <code>CSDBStrategy</code> to be used
	 *  
	 */
	public static TwoTableFormSheet getRentFormSheet(
		CountingStock cs_source,
		DataBasket db,
		UIGate uigGate,
		java.util.Comparator cmp_source,
		java.util.Comparator cmp_dest,
		boolean show_zeros,
		TableEntryDescriptor ted_source,
		TableEntryDescriptor ted_dest,
		CSDBStrategy csdbs) {

		TwoTableFormSheet ttfs =
			TwoTableFormSheet.create(
				"Choose your videos!",
				cs_source,
				db,
				uigGate,
				cmp_source,
				cmp_dest,
				show_zeros,
				ted_source,
				ted_dest,
				csdbs);

		ttfs.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				fs.removeAllButtons();
				fs.addButton("Rent", FB_RENT, null);
				fs.addButton("Cancel", FB_CANCEL, null);
			}
		});

		return ttfs;
	}
	/**
	 * @return a <code>TwoTableFormSheet</code> used to select the money the user inserts in the automat.
	 * @param c_source
	 *                  the source <code>Catalog</code> containing the possible coins
	 * @param cs_dest
	 *                  the destination <code>Stock</code>
	 * @param db
	 *                  the <code>DataBasket</code> used for the transactions
	 * @param uig
	 *                  the <code>UIGate</code> this <code>FormSheet</code> is displayed
	 * @param cmp_source
	 *                  the <code>Comparator</code> for the <code>Catalog</code>
	 * @param cmp_dest
	 *                  the <code>Comparator</code> for the <code>Stock</code>
	 * @param show_zeros
	 *                  if false, lines containing a '0' in the "Count" field of the <code>Stock</code> will be hidden.
	 * @param ted_source
	 *                  the <code>TableEntryDescriptor</code> for the <code>Catalog</code>
	 * @param ted_dest
	 *                  the <code>TableEntryDescriptor</code> for the <code>Stock</code>
	 * @param ccss
	 *                  the <code>CCSStrategy</code> to be used
	 * @param value
	 *                  a <code>String</code> -representation of the value the user has to pay
	 *  
	 */
	public static TwoTableFormSheet getPayFormSheet(
		Catalog c_source,
		CountingStock cs_dest,
		DataBasket db,
		UIGate uig,
		java.util.Comparator cmp_source,
		java.util.Comparator cmp_dest,
		boolean show_zeros,
		TableEntryDescriptor ted_source,
		TableEntryDescriptor ted_dest,
		CCSStrategy ccss,
		final String value) {

		TwoTableFormSheet ttfs =
			TwoTableFormSheet.create(
				"Throw the money in the slot, please.",
				c_source,
				cs_dest,
				db,
				uig,
				cmp_source,
				cmp_dest,
				show_zeros,
				ted_source,
				ted_dest,
				ccss);

		ttfs.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				
				JComponent jc = new JPanel();
				jc.setLayout(new BoxLayout(jc, BoxLayout.Y_AXIS));
				jc.add(Box.createVerticalStrut(10));
				jc.add(new JLabel("You have to pay: " + value));
				jc.add(Box.createVerticalStrut(10));
				jc.add(fs.getComponent());

				fs.setComponent(jc);
				fs.removeAllButtons();
				fs.addButton("Pay", FB_PAY, null);
				fs.addButton("Cancel", FB_CANCEL, null);
			}
		});
		return ttfs;
	}
	/**
	 * @return a <code>FormSheet</code> which shows the rented videos and which money the user gets back
	 * @param db
	 *                  the <code>DataBasket</code> containig the rented videos
	 * @param dbc_videos
	 *                  a <code>DataBasketCondition</code> affecting that only the rented videos in the <code>DataBasket</code>
	 *                  are shown
	 * @param ted_videos
	 *                  the <code>TableEntryDescriptor</code> for the <code>DataBasket</code>
	 * @param cs_money
	 *                  the <code>Stock</code> containing the money
	 * @param uig
	 *                  the <code>UIGate</code> this <code>FormSheet</code> is displayed
	 *  
	 */
	public static FormSheet getConfirmFormSheet(
		final DataBasket db,
		final DataBasketCondition dbc_videos,
		final TableEntryDescriptor ted_videos,
		final CountingStock cs_money,
		final UIGate uig) {
		SingleTableFormSheet stfs_videos =
			SingleTableFormSheet.create(
				"Confirm your transaction!",
				db,
				uig,
				dbc_videos,
				ted_videos);

		stfs_videos.addContentCreator(new FormSheetContentCreator() {
			public void createFormSheetContent(FormSheet fs) {
				SingleTableFormSheet stfs_money =
				SingleTableFormSheet.create("", cs_money, uig, db);
				
				JComponent jc = new JPanel();
				jc.setLayout(new BoxLayout(jc, BoxLayout.Y_AXIS));
				jc.add(new JLabel("All your rented videos:"));
				jc.add(fs.getComponent());
				jc.add(new JLabel("The money you`ll get back:"));
				jc.add(stfs_money.getComponent());
				jc.add(new JLabel("Please, click Ok to confirm the transaction!"));
				fs.setComponent(jc);
				fs.removeButton(FormSheet.BTNID_CANCEL);
			}
		});

		return stfs_videos;
	}
}
